declare module '*.json' {
  const value: any;
  export default value;
}

declare module '*.svg' {
  const value: string;
  export default value;
}

declare module '*.md' {
  const value: string;
  export default value;
}
